<?php

// Stand-alone PHP example script to use the Amazon Web Services interface
// Written by Jaap van Ganswijk (JvG) <ganswijk@xs4all.nl>, http://www.chipdir.nl/amazon/
// Updated by Klaus Burton (SubJunk), http://www.spirton.com
// You can use this version under any version of the GPL (GNU Public License)

// Send html-header so utf8's are correctly shown
echo '
	<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">
	<html>
		<head>
			<meta http-equiv="content-type" content="text/html; charset=utf-8">
		</head>
		<body>
';

$Amicrotime['total'][0] = getmicrotime();  //time at start of total process

if (!isset($page)) {
	$page = 1;
}

// Set these to correspond with your site.
// This will determine the first results that the user will see.
// The user can overrule them using the search form.
$Adefault = array(
	'language' => 'en', //what language to render the page in
	'locale' => 'us', //which server's products? available: ca,de,fr,jp,uk,us
	//'mode'               =>'books',        //what product category?
	'page' => $page, //first page to show (we are counting from 1 not 0)
	//'search'             =>'Machiavelli',  //what to search for?
	'operation' => 'ItemSearch', //what to do?
	'searchindex' => 'Books', //what product category for search?
	'searchparameter' => 'Author', //what kind of search?
	'searchparameterdata' => 'Machiavelli', //what to search for?
	//here some debugging flags you can put at the end of the URL to call this script with, like: '?show_array=true'
	'jaap' => false, //my own debugflag to test new things
	'show_array' => false, //debug: show complete incoming array? You can use this to see what other information Amazon is sending
	'show_url' => false, //debug: show XML request url to be send to Amazon?
	'show_xml' => false, //debug: show incoming XML code from Amazon?
);

require_once "data.php";      //static data about Amazon (servers, search fields) v3-
require_once "data_v4.php";   //static data about Amazon (servers, search fields) v4+
require_once "language.php";  //translate the output to the prefered language of the user
require_once "aws.php";       //see: http://mierendo.com/software/aws_signed_query/, please note that I'm using an slightly adapted version of the script: I do my own XML-interpreting

$Aassociates_id = array(
	'de' => 'chipdir00',
	'es' => 'INVALID',
	'fr' => 'chipdir010',
	'it' => 'INVALID',
	'jp' => 'INVALID',
	'uk' => 'chipdir03',
	'us' => 'chipdir'
);
//If you haven't gotten your own associate ID yet, get it from:
//http://associates.amazon.com/exec/panama/associates/apply
//
//Normally you will want to start selling books via Amazon.com only,
//just to see if it works and pays well (which it normally doesn't).
//
//You can save yourself a lot of trouble by not applying for an associate ID
//at the other local Amazon sites yet, but leaving mine in for the time being
//and perhaps I'll make a few pounds, euro's and/or yen. As soon as I would
//receive a noticable amount of cash I'll track you down an will warn you
//to get your own ID's there.
//
//please note that all those local Amazons send out all of their communications
//in their local languages, so French, German and Japanese for example and
//that you first have to earn enough money per branch before you get paid.
//And the Japanese branch only seems to pay in gift-certificates, so you'd
//have to buy Japanese books for example.
//
//if you want to disable any of the foreign branches, please do so in the
//file data.php, but please consider that we're living in an ever better
//connected world and sometime later you might want to add them again
//if only as a service to your visitors.
//other flags:
//this doesn't seem to be relevant anymore:
//$f_rich=true;       //when set to true this will convert the URL so you'll get 7.5% on qualifying books instead of 5%. It doesn't work yet outside of the USA.
//change it to false when you prefer to get only 5% ;-)
//see http://forums.prospero.com/n/mb/message.asp?webtag=am-assosdev&msg=6.1&ctx=0, or search the forum for 'nosim'
//This is only helpful for certain books
//correction: keep this option set to true, to prevent the showing of Amazon's annoying distraction pages.
//20080530: i don't know if this is still relevant for version 4
$f_user_search = true;  //add a user search form and allow user searches?
//=============================================================================
//==You don't have to change anything below this line to start earning money.==
//=============================================================================

//what are the user's prefered languages?
$Ahttp_lang = split(',', $_SERVER['HTTP_ACCEPT_LANGUAGE']);

//try to find the visitor's prefered language, this is still experimental
foreach ($Ahttp_lang as $i => $d) {
	if (isset($Axl[$l = substr($d, 0, 2)])) {
		$pref_lang = $l;
		break;
	}
}

if (isset($pref_lang)) {
	$Adefault['language'] = $pref_lang;
}

//for all parameters see if the user has overruled it or use the default
foreach ($Adefault as $i => $d) {
	$$i = isset($_GET[$i]) ? $_GET[$i] : $d;
}

if (!isset($search)) {
	$search = $Adefault['searchparameterdata'];
}

$parameters = array(
	'Operation' => $operation,
	'Keywords' => urlencode($search),
	'SearchIndex' => $searchindex, //Books for example.
	"$searchparameter" => $searchparameterdata,
	'ItemPage' => $page, //which page?
	'AssociateTag' => $Aassociates_id[$locale],
	'ResponseGroup' => 'Medium', //Small, Medium, Large or SellerListing
);

$A = aws_request($parameters);  //do the AWS request via cache and decode the XML

$Amicrotime['XML'][1] = getmicrotime();	//time after XML decoding

//show the stuff

$Amicrotime['show'][0] = getmicrotime();	 //time before showing stuff

echo "<h2>" . xu('searching for') . " '$searchparameterdata'</h2>\n";  //you can change the h2's to h1's

if (isset($A->Error)) {
	$c = $A->Error->Code;
	$e = $A->Error->Message;
	echo "Amazon error: $c: $e<br>\n";
} else if ($A->Items->Request->IsValid != 'True') {
	$c = $A->Items->Request->Errors->Error->Code;
	$e = $A->Items->Request->Errors->Error->Message;
	echo "Amazon error: $c: $e<br>\n";
} else {
	print_products($A);
}

//print all the products
function print_products($A) {
	global $language;
	global $locale;
	global $operation;
	global $page;
	global $search;
	global $searchparameter;
	global $searchparameterdata;

	switch ($operation) {
		case 'SellerListingSearch':
			die("sorry but SellerListingSearch doesn't work yet");
			$items = $A['SellerListingSearchResponse']['SellerListings'];
			$subnode = 'SellerListing';
			break;
		default:
			$items = $A->Items;
			$aitems = (array) $items; //it is very strange that this and the next line works and not the outcommented line before this one
			$items_sub = $aitems['Item'];
			break;
	}
	$npro = $items->TotalResults;
	$npag = $items->TotalPages;
	echo "<b>" . xu('number of products') . ": $npro</b><br>\n";
	echo "<b>" . xu('page') . ' ' . ($page) . ' ' . xl('of') . ' ' . ((int) $npag) . "</b><br>\n";
	echo "<p />\n";
	echo "<table border='1'>\n";
	echo "  <tbody>\n";
	if (is_array($items_sub) and $items_sub[0]) {
		foreach ($items_sub as $i => $E) {
			print_product($E, '    ');
		}
	} else {
		print_product($D, '    ');
	}
	echo "  </tbody>\n";
	echo "</table>\n";
	//provide links to previous and next page
	if ($npag > 1) {
		echo xu('page:') . "\n";
		echo make_link_to_page(1, 1) . "\n";
		echo make_link_to_page(2, 2) . "\n";
		if ($page >= 2) {
			echo make_link_to_page($page - 1, '<') . "\n";
		}
		if ($page < $npag) {
			echo make_link_to_page($page + 1, '>') . "\n";
		}
		echo make_link_to_page($npag, $npag) . "\n";
	}
}

//print one Amazon product
function print_product($E) {
	global $locale;

	if ($E) {
		$url = $E->DetailPageURL;
		$E2A = (array) $E;
		$url = $E2A['DetailPageURL'];

		echo "<tr>\n";
		echo "  <td align='center'>\n";
		if ($d = $E->MediumImage) {
			$iu = $d->URL;
			$ih = $d->Height;
			$iw = $d->Width;
			echo "    <a href='$url'><img src='$iu' width='$iw' height='$ih' border='0'></a>\n";
		}
		echo "  </td>\n";
		echo "  <td align='center'>\n";
		if ($d = $E->ItemAttributes->Title) {
			echo "<a href='$url'>$d</a><br>\n";
		}
		if (is_array($dd = $E->ItemAttributes->Author)) {
			echo "    " . xl('by') . ": ";
			foreach ($dd as $i => $d) {
				if ($i)
					echo ', ';
				echo "<b>$d</b>";
			}
			echo "<br>\n";
		}
		else if ($d = $dd) {
			echo "    " . xl('by') . ": <b>$d</b><br>\n";
		}
		$dm = $E->ItemAttributes->Publisher;
		$dr = $E->ItemAttributes->PublicationDate;
		if ($dm or $dr) {
			echo "    ";
			if ($dm) {
				echo xl('publisher') . ": <b>$dm</b>";
			}
			if ($dm and $dr)
				echo ', ';
			if ($dr) {
				echo xl('published') . ": <b>$dr</b>";
			}
			echo "<br />\n";
		}
		if ($asin = $E->ASIN) {
			echo "    ASIN: <b>$asin</b><br>\n";
		}
		if ($ean = $E->ItemAttributes->EAN) {
			echo "    EAN: <b>$ean</b><br>\n";
		}
		$da = false;  //don't show
		$ds = $E->SalesRank;
		if ($da or $ds) {
			if ($ds)
				echo xl('sales rank') . ": <b>$ds</b>";
			if ($da and $ds)
				echo ", ";
			if ($da)
				echo xl('customer rating') . ": <b>" . (($da - 1) * 25) . "%</b>";
			echo "<br>\n";
		}
		$do = false;
		$du = false;
		if (!empty($E->OfferSummary->LowestNewPrice->FormattedPrice)) {
			$do = $E->OfferSummary->LowestNewPrice->FormattedPrice;
		}
		if (!empty($E->OfferSummary->LowestUsedPrice->FormattedPrice)) {
			$du = $E->OfferSummary->LowestUsedPrice->FormattedPrice;
		}

		if ($do or $du) {
			echo xl('price') . ": ";
			if ($do)
				echo "<b>$do</b> (" . xl('new') . ")";
			if ($do and $du)
				echo ", ";
			if ($du)
				echo "<b>$du</b> (" . xl('used') . ")";
			echo "<br>\n";
		}
		echo "  </td>\n";
		if ($EditorialReviews = $E->EditorialReviews and
			$EditorialReview = $EditorialReviews->EditorialReview and
			$Content = $EditorialReview->Content) {
			echo "  <td align='center'>\n";
			echo "    $Content\n";
			echo "  </td>\n";
		}
		echo "</tr>\n";
	}
}

//make link to other page (next, previous etc.)
function make_link_to_page($page, $s) {
	global $language;
	global $locale;
	global $search;
	global $searchindex;
	global $searchparameter;
	global $searchparameterdata;

	return "<a href='?" .
		'language' . '=' . $language . '&' .
		'locale' . '=' . $locale . '&' .
		'page' . '=' . $page . '&' .
		'searchindex' . '=' . $searchindex . '&' .
		'searchparameter' . '=' . $searchparameter . '&' .
		'searchparameterdata' . '=' . urlencode($searchparameterdata) . '&' .
		"'>" . xu($s) . "</a>";
}

//print an array or object
//(for debugging)
function print_array($A) {
	if (is_object($A)) {
		echo "object:<br >\n";
		print_array((array) $A);
	} else if (is_array($A)) {
		echo "<table border='1'>\n";
		echo "  <tbody>\n";
		foreach ($A as $i => $d) {
			echo "    <tr>\n";
			echo "      <td valign='top'>$i</td>\n";
			if (is_string($d)) {
				echo "      <td>" . htmlentities($d) . "</td>\n";
			} else if (is_array($d)) {
				echo "      <td>\n";
				print_array($d, '        ');
				echo "      </td>\n";
			} else if (is_object($d)) {
				echo "      <td>object:\n";
				print_array((array) $d, '        ');
				echo "      </td>\n";
			} else {
				echo "      <td>" . htmlentities($d) . "</td>\n";
			}
			echo "    </tr>\n";
		}
		echo "</tbody>\n";
		echo "</table>\n";
	} else {
		echo "warning: not-an-array-or-object given to print_array()<br>\n";
	}
}

function getmicrotime() {
	list($us, $s) = explode(' ', microtime());
	return (float) $us + (float) $s;
}

if ($f_user_search) {
	//the next section adds a search box for the users
	echo "<hr>\n";
	echo "<form method='get' action='?'>\n";
	echo "  <table>\n";
	echo "    <tbody>\n";
	echo "      <tr>\n";
	echo "        <td>" . xu('language for page:') . "</td>\n";
	echo "        <td>\n";
	echo "          <select name='language'>\n";
	foreach ($Alanguage as $i => $d) {
		echo "            <option value='$i'" . ($i == $language ? " selected='true'" : "") . ">" . xu($d) . "</option>\n";
	}
	echo "          </select>\n";
	echo "        </td>\n";
	echo "      </tr>\n";
	echo "      <tr>\n";
	echo "        <td>" . xu('which server:') . "</td>\n";
	echo "        <td>\n";
	echo "          <select name='locale'>\n";
	foreach ($Alocale as $i => $d) {
		echo "            <option value='$i'" . ($i == $locale ? " selected='true'" : "") . ">" . xl($d) . "</option>\n";
	}
	echo "          </select>\n";
	echo "        </td>\n";
	echo "      </tr>\n";
	echo "      <tr>\n";
	echo "        <td>" . xu('operation:') . "</td>\n";
	echo "        <td>\n";
	echo "          <select name='operation'>\n";
	foreach ($Aoperation as $i => $d) {
		echo "            <option value='$i'" . ($i == $operation ? " selected='true'" : "") . ">" . xu($i) . "</option>\n";
	}
	echo "          </select>\n";
	echo "        </td>\n";
	echo "      </tr>\n";
	echo "      <tr>\n";
	echo "        <td>" . xu('product category:') . "</td>\n";
	echo "        <td>\n";
	echo "          <select name='searchindex'>\n";
	foreach ($Asearchindex as $i => $d) {  //uk and jp not ready yet? 20080730: must be adjusted to the locale again
		echo "            <option value='$d'" . ($d == $searchindex ? " selected='true'" : "") . ">" . xl($d) . "</option>\n";
	}
	echo "          </select>\n";
	echo "        </td>\n";
	echo "      </tr>\n";
	echo "      <tr>\n";
	echo "        <td>" . xu('search parameter:') . "</td>\n";
	echo "        <td>\n";
	echo "          <select name='searchparameter'>\n";
	foreach ($Aoperation['ItemSearch']['RequiredParameters']['SearchIndex'] as $i => $d) {
		echo "            <option value='$d'" . ($d == $searchparameter ? " selected='true'" : "") . ">" . xu($d) . "</option>\n";
	}
	echo "          </select>\n";
	echo "        </td>\n";
	echo "      </tr>\n";
	echo "      <tr>\n";
	echo "        <td>" . xu('search string:') . "</td>\n";
	echo "        <td><input type='text' name='searchparameterdata' value='$searchparameterdata' /></td>\n";
	echo "      </tr>\n";
	echo "    </tbody>\n";
	echo "  </table>\n";
	echo "  <input type='submit' value='Search' />\n";
	echo "</form>\n";
	//end of the search box section
}

$Amicrotime['show'][1] = getmicrotime(); // Time after showing
$Amicrotime['total'][1] = getmicrotime(); // Time at end

// The next part is to show if the script is overburdoning your server. The time waiting for Amazon isn't burdoning your server.
echo "<hr>\n";
echo "Time used:<br>\n";
foreach ($Amicrotime as $i => $d) {
	printf("- %.2f s %s<br>\n", $d[1] - $d[0], $i);
}

// You may remove these lines or move them to an about section of your site:
echo "<hr>\n";
echo "This script is GPL'ed and can be downloaded from <a href='http://www.spirton.com/'>this page</a>.<br>It is based on the script from <a href='http://www.chipdir.nl/amazon/'>this page</a>.\n";

echo "  </body>\n";
echo "</html>";
